var Immutable, LiteJSONViewer, React, Viewer, classnames, diff, div, isSafari, keycode, locale, pre, recorder, ref, scrollbar, style, tabs;

diff = require('immutablediff');

React = require('react');

keycode = require('keycode');

Immutable = require('immutable');

classnames = require('classnames');

recorder = require('./recorder');

locale = require('./app/locale');

scrollbar = require('./app/scrollbar');

isSafari = require('./app/is-safari');

LiteJSONViewer = React.createFactory(require('react-lite-json-viewer'));

Viewer = React.createFactory(require('./app/viewer'));

ref = React.DOM, div = ref.div, pre = ref.pre, style = ref.style;

tabs = ['action', 'store', 'diff'];

module.exports = React.createClass({
  displayName: "actions-recorder-devtools",
  propTypes: {
    core: React.PropTypes.instanceOf(Immutable.Map).isRequired,
    language: React.PropTypes.string,
    dispatch: React.PropTypes.func,
    width: React.PropTypes.number.isRequired,
    height: React.PropTypes.number.isRequired,
    path: React.PropTypes.instanceOf(Immutable.List).isRequired,
    onPathChange: React.PropTypes.func.isRequired
  },
  getDefaultProps: function() {
    return {
      language: 'en'
    };
  },
  getInitialState: function() {
    return {
      tab: 'action'
    };
  },
  getStoreAtPointer: function(pointer) {
    var core, updater;
    core = this.props.core;
    if (pointer < 1) {
      return core.get('initial');
    } else {
      updater = (function(_this) {
        return function(acc, record) {
          return core.get('updater')(acc, record.get(0), record.get(1));
        };
      })(this);
      return core.get('records').slice(0, pointer).reduce(updater, core.get('initial'));
    }
  },
  dispatch: function(actionName, actionData) {
    if (this.props.dispatch != null) {
      return this.props.dispatch(actionName, actionData);
    } else {
      return recorder.dispatch(actionName, actionData);
    }
  },
  onTabSelect: function(name) {
    return this.setState({
      tab: name
    });
  },
  onCommit: function() {
    return this.dispatch("actions-recorder/commit");
  },
  onReset: function() {
    return this.dispatch("actions-recorder/reset");
  },
  onPeek: function(position) {
    return this.dispatch("actions-recorder/peek", position);
  },
  onRun: function() {
    return this.dispatch("actions-recorder/run");
  },
  onMergeBefore: function() {
    if (this.props.core.get('pointer') > 0) {
      return this.dispatch("actions-recorder/merge-before");
    }
  },
  onClearAfter: function() {
    return this.dispatch("actions-recorder/clear-after");
  },
  onStep: function(event) {
    return this.dispatch('actions-recorder/step', event.shiftKey);
  },
  onInitialClick: function() {
    return this.dispatch('actions-recorder/peek', 0);
  },
  renderItem: function(record, originalIndex) {
    var core, index, isSelected, onClick;
    core = this.props.core;
    index = core.get('records').size - originalIndex;
    onClick = (function(_this) {
      return function() {
        return _this.onPeek(index);
      };
    })(this);
    isSelected = core.get('pointer') === index && core.get('isTravelling');
    return div({
      key: index,
      style: this.styleItem(isSelected),
      onClick: onClick
    }, record.get(0));
  },
  renderAction: function() {
    var actionData, record;
    if (this.props.core.get('pointer') > 0) {
      record = this.props.core.get('records').get(this.props.core.get('pointer') - 1);
      actionData = record.get(1);
    } else {
      actionData = null;
    }
    return LiteJSONViewer({
      key: this.state.tab,
      height: this.props.height - 70,
      data: actionData,
      path: this.props.path,
      onChange: this.props.onPathChange
    });
  },
  renderStore: function() {
    var result;
    result = this.getStoreAtPointer(this.props.core.get('pointer'));
    return LiteJSONViewer({
      key: this.state.tab,
      height: this.props.height - 70,
      data: result,
      path: this.props.path,
      onChange: this.props.onPathChange
    });
  },
  renderDiff: function() {
    var changes, core, error, prevResult, result;
    core = this.props.core;
    result = this.getStoreAtPointer(core.get('pointer'));
    prevResult = this.getStoreAtPointer(core.get('pointer') - 1);
    try {
      changes = diff(prevResult, result);
    } catch (_error) {
      error = _error;
      changes = error;
    }
    return LiteJSONViewer({
      key: this.state.tab,
      height: this.props.height - 70,
      data: changes,
      path: this.props.path,
      onChange: this.props.onPathChange
    });
  },
  renderCurrent: function() {
    var core, store;
    core = this.props.core;
    store = core.get('store');
    return LiteJSONViewer({
      key: this.state.tab,
      height: this.props.height - 70,
      data: store,
      path: this.props.path,
      onChange: this.props.onPathChange
    });
  },
  renderDetails: function() {
    return div({
      style: this.styleDetails()
    }, div({
      style: this.styleMode()
    }, tabs.map((function(_this) {
      return function(name) {
        var onClick;
        onClick = function() {
          return _this.onTabSelect(name);
        };
        return div({
          style: _this.styleEntry(name === _this.state.tab),
          onClick: onClick,
          key: name
        }, name);
      };
    })(this))), (function() {
      switch (this.state.tab) {
        case 'action':
          return this.renderAction();
        case 'store':
          return this.renderStore();
        case 'diff':
          return this.renderDiff();
      }
    }).call(this));
  },
  render: function() {
    var core, hint, isInitialSelected, isTravelling, records, showClearAfter, showCommit, showMergeBefore, showReset, showRun, showStep;
    core = this.props.core;
    hint = (function(_this) {
      return function(text) {
        return locale.get(text, _this.props.language);
      };
    })(this);
    isInitialSelected = core.get('pointer') === 0 && core.get('isTravelling');
    records = core.get('records');
    isTravelling = core.get('isTravelling');
    showMergeBefore = core.get('pointer') > 0 && isTravelling;
    showClearAfter = core.get('pointer') < records.size && isTravelling;
    showReset = records.size > 0;
    showCommit = records.size > 0;
    showRun = isTravelling;
    showStep = isTravelling && records.size > 0;
    return div({
      style: this.styleRoot(),
      className: 'actions-recorder-devtools'
    }, style(null, scrollbar), div({
      style: this.styleHeader()
    }, div({
      style: this.styleButton(showMergeBefore),
      onClick: this.onMergeBefore
    }, hint('mergeBefore')), div({
      style: this.styleButton(showClearAfter),
      onClick: this.onClearAfter
    }, hint('clearAfter')), div({
      style: this.styleButton(showReset),
      onClick: this.onReset
    }, hint('reset')), div({
      style: this.styleButton(showCommit),
      onClick: this.onCommit
    }, hint('commit')), div({
      style: this.styleButton(showRun),
      onClick: this.onRun
    }, hint('run')), div({
      style: this.styleButton(showStep),
      onClick: this.onStep
    }, hint('step')), div({
      style: this.styleTip()
    }, "(" + (core.get('pointer')) + "/" + (core.get('records').size) + ")")), div({
      style: this.styleViewer()
    }, div({
      style: this.styleMonitor()
    }, core.get('records').reverse().map(this.renderItem), div({
      style: this.styleItem(isInitialSelected),
      onClick: this.onInitialClick
    }, '__initial__')), core.get('isTravelling') && (core.get('records').get(core.get('pointer') - 1) != null) ? this.renderDetails() : this.renderCurrent()));
  },
  styleRoot: function() {
    return {
      background: 'hsla(0, 0%, 0%, 0.56)',
      color: 'white',
      fontFamily: 'Menlo, Consolas, Ubuntu Mono, monospace',
      lineHeight: '1.8em',
      display: isSafari ? '-webkit-flex' : 'flex',
      height: '100%',
      flexDirection: 'column',
      WebkitFlexDirection: 'column',
      transitionProperty: 'left, top',
      transitionDuration: '300ms',
      zIndex: 9999,
      flex: 1,
      WebkitFlex: 1
    };
  },
  styleButton: function(isAvailable) {
    return {
      display: 'inline-block',
      backgroundColor: isAvailable ? 'hsla(0,100%,100%,0.2)' : 'hsla(0,100%,80%,0.2)',
      color: isAvailable ? 'white' : 'hsla(0,100%,100%,0.4)',
      marginRight: '10px',
      padding: '0 10px',
      fontSize: '12px',
      fontFamily: 'Verdana, Helvetica, sans-serif',
      lineHeight: '30px',
      cursor: 'pointer'
    };
  },
  styleEntry: function(isSelected) {
    return {
      display: 'inline-block',
      backgroundColor: isSelected ? 'hsla(0,100%,100%,0.5)' : 'hsla(0,100%,100%,0.2)',
      marginRight: '10px',
      padding: '0 10px',
      fontSize: '12px',
      fontFamily: 'Verdana, Helvetica, sans-serif',
      lineHeight: '30px',
      cursor: 'pointer'
    };
  },
  styleHeader: function() {
    return {
      WebkitUserSelect: 'none',
      marginBottom: '10px'
    };
  },
  styleViewer: function() {
    return {
      flex: 1,
      WebkitFlex: 1,
      display: isSafari ? '-webkit-flex' : 'flex',
      flexDirection: 'row',
      WebkitFlexDirection: 'row',
      height: this.props.height - 40,
      position: 'relative'
    };
  },
  styleMonitor: function() {
    return {
      width: 'auto',
      paddingRight: '40px',
      marginRight: '10px',
      overflowX: 'hidden',
      paddingTop: '100px',
      paddingBottom: '100px',
      height: this.props.height - 40
    };
  },
  styleItem: function(isPointer) {
    return {
      cursor: 'pointer',
      lineHeight: '30px',
      fontSize: '12px',
      padding: '0 10px',
      backgroundColor: isPointer ? 'hsla(0,100%,100%,0.2)' : 'transparent',
      whiteSpace: 'nowrap'
    };
  },
  styleDetails: function() {
    return {
      flex: 1,
      WebkitFlex: 1,
      overflow: 'auto',
      display: isSafari ? '-webkit-flex' : 'flex',
      flexDirection: 'column',
      WebkitFlexDirection: 'column'
    };
  },
  styleMode: function() {
    return {
      height: '30px'
    };
  },
  styleTip: function() {
    return {
      fontSize: '12px',
      display: 'inline-block'
    };
  }
});
