var Immutable, callUpdater, core, getStoreFrom, recorderEmit, recorderListeners;

Immutable = require("immutable");

core = Immutable.Map({
  records: Immutable.List(),
  pointer: 0,
  isTravelling: false,
  initial: Immutable.Map(),
  store: Immutable.Map(),
  updater: function(state) {
    return state;
  },
  inProduction: false
});

recorderListeners = Immutable.List();

recorderEmit = function() {
  return recorderListeners.forEach(function(fn) {
    return fn(core);
  });
};

getStoreFrom = function(records) {
  var updateHandler, updater;
  updateHandler = core.get('updater');
  updater = function(acc, action) {
    return updateHandler(acc, action.get(0), action.get(1));
  };
  return records.reduce(updater, core.get('initial'));
};

callUpdater = function(actionType, actionData) {
  var actionName, groupName, newRecords, newStore, nextPointer, pointer, records, ref, updateHandler;
  pointer = core.get('pointer');
  records = core.get('records');
  ref = actionType.split("/"), groupName = ref[0], actionName = ref[1];
  if (groupName === "actions-recorder") {
    switch (actionName) {
      case "commit":
        newStore = getStoreFrom(records);
        return {
          initial: newStore,
          store: newStore,
          records: Immutable.List(),
          pointer: 0,
          isTravelling: false
        };
      case "reset":
        return {
          records: Immutable.List(),
          pointer: 0,
          isTravelling: false,
          store: core.get('initial')
        };
      case "peek":
        nextPointer = actionData;
        if (nextPointer === 0) {
          return {
            pointer: 0,
            isTravelling: true,
            store: core.get('initial')
          };
        } else {
          return {
            pointer: nextPointer,
            isTravelling: true,
            store: getStoreFrom(records.slice(0, nextPointer))
          };
        }
        break;
      case "run":
        return {
          isTravelling: false,
          pointer: 0,
          store: getStoreFrom(records)
        };
      case "merge-before":
        if (pointer <= 1) {
          return {};
        } else {
          newStore = getStoreFrom(records.slice(0, pointer));
          return {
            initial: newStore,
            records: records.slice(pointer - 1),
            pointer: 1,
            store: newStore
          };
        }
        break;
      case "clear-after":
        if (pointer === records.size) {
          return {};
        } else {
          newRecords = records.slice(0, pointer);
          return {
            records: newRecords,
            store: getStoreFrom(newRecords)
          };
        }
        break;
      case 'step':
        if (actionData) {
          if (pointer > 0) {
            nextPointer = pointer - 1;
            return {
              pointer: nextPointer,
              store: getStoreFrom(records.slice(0, nextPointer))
            };
          } else {
            return {};
          }
        } else {
          if (pointer < records.size) {
            nextPointer = pointer + 1;
            return {
              pointer: nextPointer,
              store: getStoreFrom(records.slice(0, nextPointer))
            };
          } else {
            return {};
          }
        }
        break;
      default:
        console.warn("Unknown actions-recorder action: " + actionType);
        return {};
    }
  } else {
    newRecords = records.push(Immutable.List([actionType, actionData]));
    if (core.get('isTravelling')) {
      return {
        records: newRecords
      };
    } else {
      updateHandler = core.get('updater');
      return {
        records: newRecords,
        store: updateHandler(core.get('store'), actionType, actionData)
      };
    }
  }
};

exports.setup = function(options) {
  core = core.merge(Immutable.fromJS(options)).set('store', options.initial);
  if (core.get('inProduction')) {
    return setInterval(function() {
      if (core.get('records').size > 40 && (!core.get('isTravelling'))) {
        return exports.dispatch('actions-recorder/commit');
      }
    }, 5 * 60 * 1000);
  }
};

exports.hotSetup = function(options) {
  core = core.merge(Immutable.fromJS(options)).set('store', getStoreFrom(core.get('records')));
  return recorderEmit();
};

exports.request = function(fn) {
  return fn(core);
};

exports.getState = function() {
  return exports.getStore();
};

exports.getStore = function() {
  return core.get('store');
};

exports.subscribe = function(fn) {
  return recorderListeners = recorderListeners.unshift(fn);
};

exports.unsubscribe = function(fn) {
  return recorderListeners = recorderListeners.filterNot(function(listener) {
    return listener === fn;
  });
};

exports.dispatch = function(actionType, actionData) {
  actionData = Immutable.fromJS(actionData);
  core = core.merge(callUpdater(actionType, actionData));
  return recorderEmit();
};
